@@ -274,9 +274,7 @@ class OBSSceneItem(Reference[obs_sceneitem_t]):
     @property
     def blending_method(self) -> OBSBlendingMethod:
         with self as scene_item:
-            return OBSBlendingMethod(
-                obspython.obs_sceneitem_get_blending_method(scene_item)
-            )
+            return OBSBlendingMethod(obspython.obs_sceneitem_get_blending_method(scene_item))
 
     @blending_method.setter
     def blending_method(self, value: OBSBlendingMethod) -> None:
@@ -286,9 +284,7 @@ class OBSSceneItem(Reference[obs_sceneitem_t]):
     @property
     def blending_mode(self) -> OBSBlendingType:
         with self as scene_item:
-            return OBSBlendingType(
-                obspython.obs_sceneitem_get_blending_mode(scene_item)
-            )
+            return OBSBlendingType(obspython.obs_sceneitem_get_blending_mode(scene_item))
 
     @blending_mode.setter
     def blending_mode(self, value: OBSBlendingType) -> None:
@@ -470,9 +466,11 @@ class OBSScene(Reference[obs_scene_t]):
         obs_scene = obspython.obs_scene_duplicate(self, name, duplicate_type.value)
         return OBSScene(obs_scene)
 
-    def find_source(self, name: str) -> OBSSceneItem:
+    def find_source(self, name: str) -> OBSSceneItem | None:
         with self as scene:
             obs_sceneitem = obspython.obs_scene_find_source(scene, name)
+        if obs_sceneitem is None:
+            return
         return OBSSceneItem(obs_sceneitem)
 
     def find_source_recursive(self, name: str) -> OBSSceneItem:
@@ -513,9 +511,7 @@ class OBSScene(Reference[obs_scene_t]):
 
     def insert_group(self, name: str, items: list[OBSSceneItem]) -> OBSSceneItem:
         with self as scene:
-            obs_sceneitem = obspython.obs_scene_insert_group(
-                scene, name, items, len(items)
-            )
+            obs_sceneitem = obspython.obs_scene_insert_group(scene, name, items, len(items))
         return OBSSceneItem(obs_sceneitem)
 
     def add_group2(self, name: str, signal: bool) -> OBSSceneItem:
@@ -523,13 +519,9 @@ class OBSScene(Reference[obs_scene_t]):
             obs_sceneitem = obspython.obs_scene_add_group2(scene, name, signal)
         return OBSSceneItem(obs_sceneitem)
 
-    def insert_group2(
-        self, name: str, items: list[OBSSceneItem], signal: bool
-    ) -> OBSSceneItem:
+    def insert_group2(self, name: str, items: list[OBSSceneItem], signal: bool) -> OBSSceneItem:
         with self as scene:
-            obs_sceneitem = obspython.obs_scene_insert_group2(
-                scene, name, items, len(items), signal
-            )
+            obs_sceneitem = obspython.obs_scene_insert_group2(scene, name, items, len(items), signal)
         return OBSSceneItem(obs_sceneitem)
 
     def get_group(self, name: str) -> OBSSceneItem:
@@ -546,9 +538,7 @@ class OBSScene(Reference[obs_scene_t]):
 
     def sceneitem_from_source(self, source: OBSSource) -> OBSSceneItem:
         with self as scene, source as source_data:
-            obs_sceneitem = obspython.obs_scene_sceneitem_from_source(
-                scene, source_data
-            )
+            obs_sceneitem = obspython.obs_scene_sceneitem_from_source(scene, source_data)
         return OBSSceneItem(obs_sceneitem)
 
     @property
