@@ -1,34 +1,33 @@
 from __future__ import annotations
 
 import asyncio
-from threading import Thread
+import threading
+from pathlib import Path
 
 from loguru import logger
-from omu.app import App
+from omu.address import Address
 from omu.helper import map_optional
 from omu.omu import Omu
 from omu.token import JsonTokenProvider
 from omuplugin_obs.version import VERSION
 
-from ..const import PLUGIN_ID
 from ..obs.data import OBSData
 from ..obs.obs import OBS, OBSFrontendEvent
-from ..obs.scene import (
-    OBSBlendingMethod,
-    OBSBlendingType,
-    OBSScaleType,
-    OBSScene,
-    OBSSceneItem,
-)
+from ..obs.scene import OBSBlendingMethod, OBSBlendingType, OBSScaleType, OBSScene, OBSSceneItem
 from ..obs.source import OBSSource
 from ..types import (
+    BROWSER_ADD,
+    BROWSER_CREATE,
     EVENT_SIGNAL,
+    SCENE_CREATE,
     SCENE_GET_BY_NAME,
     SCENE_GET_BY_UUID,
     SCENE_GET_CURRENT,
     SCENE_LIST,
     SCENE_SET_CURRENT_BY_NAME,
     SCENE_SET_CURRENT_BY_UUID,
+    SCREENSHOT_CREATE,
+    SCREENSHOT_GET_LAST_BINARY,
     SOURCE_ADD,
     SOURCE_CREATE,
     SOURCE_GET_BY_NAME,
@@ -39,11 +38,14 @@ from ..types import (
     SOURCE_UPDATE,
     BlendProperties,
     BrowserSourceData,
+    CreateBrowserRequest,
     CreateResponse,
     RemoveByNameRequest,
     RemoveByUuidRequest,
     RemoveResponse,
     ScaleProperties,
+    SceneCreateRequest,
+    SceneCreateResponse,
     SceneGetByNameRequest,
     SceneGetByUuidRequest,
     SceneGetCurrentRequest,
@@ -53,6 +55,10 @@ from ..types import (
     SceneSetCurrentByNameRequest,
     SceneSetCurrentByUuidRequest,
     SceneSetCurrentResponse,
+    ScreenshotCreateRequest,
+    ScreenshotCreateResponse,
+    ScreenshotGetLastBinaryRequest,
+    ScreenshotGetLastBinaryResponse,
     SourceGetByNameRequest,
     SourceGetByUuidRequest,
     SourceJson,
@@ -60,29 +66,14 @@ from ..types import (
     TextSourceData,
     UpdateResponse,
 )
-from .config import get_token_path
-
-APP = App(
-    PLUGIN_ID,
-    version=VERSION,
-    metadata={
-        "locale": "en",
-        "name": {
-            "ja": "OBSプラグイン",
-            "en": "OBS Plugin",
-        },
-        "description": {
-            "ja": "アプリがOBSを制御するためのプラグイン",
-            "en": "Plugin for app to control OBS",
-        },
-    },
-)
+from .config import APP, get_config, get_token_path
 
 global loop
 loop = asyncio.new_event_loop()
 omu = Omu(
     APP,
     loop=loop,
+    address=Address(**get_config()["server"]),
     token=JsonTokenProvider(get_token_path()),
 )
 
@@ -93,9 +84,7 @@ def source_to_json(scene_item: OBSSceneItem) -> SourceJson | None:
         blending_method=scene_item.blending_method.name,
         blending_mode=scene_item.blending_mode.name,
     )
-    scale_properties = ScaleProperties(
-        scale_filter=scene_item.scale_filter.name,
-    )
+    scale_properties = ScaleProperties(scale_filter=scene_item.scale_filter.name)
     if source.id == "browser_source":
         return {
             "type": "browser_source",
@@ -133,14 +122,14 @@ def get_scene(scene_name: str | None) -> OBSScene:
     return scene
 
 
-def get_name(name: str) -> str:
+def get_unique_name(name: str) -> str:
     existing_source = OBSSource.get_source_by_name(name)
     if existing_source is not None:
+        removed = existing_source.removed
         existing_source.release()
-        if not existing_source.removed:
-            raise ValueError(f"Source with name {name} already exists")
-        return name
-    i = 1
+        if removed:
+            return name
+    i = 2
     while True:
         new_name = f"{name} ({i})"
         existing_source = OBSSource.get_source_by_name(new_name)
@@ -154,6 +143,11 @@ def create_obs_source(scene: OBSScene, source_json: SourceJson) -> OBSSceneItem:
     if "uuid" in source_json:
         raise NotImplementedError("uuid is not supported yet")
     settings = map_optional(dict(source_json.get("data")), OBSData.from_json)
+    if source_json["type"] not in {
+        "browser_source",
+        "text_gdiplus",
+    }:
+        raise ValueError(f"Source with type {source_json['type']} is not allowed")
     obs_source = OBSSource.create(
         source_json["type"],
         source_json["name"],
@@ -200,7 +194,7 @@ async def source_create(source: SourceJson) -> CreateResponse:
 
 @omu.endpoints.bind(endpoint_type=SOURCE_ADD)
 async def source_add(source: SourceJson) -> CreateResponse:
-    source["name"] = get_name(source["name"])
+    source["name"] = get_unique_name(source["name"])
     scene = get_scene(source.get("scene"))
     scene_item = create_obs_source(scene, source)
     scene.release()
@@ -218,6 +212,135 @@ async def source_add(source: SourceJson) -> CreateResponse:
     return {"source": source_json}
 
 
+def calculate_dimensions(scene: OBSScene, width: int | str, height: int | str) -> tuple[int, int]:
+    scene_width = scene.source.base_width
+    scene_height = scene.source.base_height
+    # 100:% = full width/height of the scene
+    # 100:vw = full width of the scene
+    # 100:vh = full height of the scene
+    # 100:px = 100 pixels
+    # 100 = raise ValueError
+    if isinstance(width, int):
+        width = int(width)
+    elif isinstance(width, str):
+        if width.count(":") != 1:
+            raise ValueError(f"Invalid width format: {width}")
+        num, unit = width.split(":")
+        if unit == "%":
+            width = int(num) * scene_width // 100
+        elif unit == "vw":
+            width = int(num) * scene_width // 100
+        elif unit == "vh":
+            width = int(num) * scene_height // 100
+        elif unit == "px":
+            width = int(num)
+        else:
+            raise ValueError(f"Unknown unit {unit} in width {width}")
+    else:
+        raise ValueError(f"Invalid width format: {width}")
+    if isinstance(height, int):
+        height = int(height)
+    elif isinstance(height, str):
+        if height.count(":") != 1:
+            raise ValueError(f"Invalid height format: {height}")
+        num, unit = height.split(":")
+        if unit == "%":
+            height = int(num) * scene_height // 100
+        elif unit == "vw":
+            height = int(num) * scene_width // 100
+        elif unit == "vh":
+            height = int(num) * scene_height // 100
+        elif unit == "px":
+            height = int(num)
+        else:
+            raise ValueError(f"Unknown unit {unit} in height {height}")
+    else:
+        raise ValueError(f"Invalid height format: {height}")
+    return width, height
+
+
+@omu.endpoints.bind(endpoint_type=BROWSER_CREATE)
+async def browser_create(browser_source: CreateBrowserRequest) -> CreateResponse:
+    existing_source = OBSSource.get_source_by_name(browser_source["name"])
+    if existing_source is not None:
+        existing_source.release()
+        if not existing_source.removed:
+            raise ValueError(f"Source with name {browser_source['name']} already exists")
+    scene = get_scene(browser_source.get("scene"))
+    scene_width = scene.source.base_width
+    scene_height = scene.source.base_height
+    width, height = calculate_dimensions(
+        scene,
+        browser_source.get("width", scene_width),
+        browser_source.get("height", scene_height),
+    )
+    obs_data = {
+        "url": browser_source["url"],
+        "width": width,
+        "height": height,
+        "css": browser_source.get("css"),
+    }
+    obs_source = OBSSource.create(
+        "browser_source",
+        browser_source["name"],
+        OBSData.from_json(obs_data),
+    )
+    scene_item = scene.add(obs_source)
+    if "blend_properties" in browser_source:
+        blending_method = browser_source["blend_properties"]["blending_method"]
+        blending_mode = browser_source["blend_properties"]["blending_mode"]
+        scene_item.blending_method = OBSBlendingMethod[blending_method]
+        scene_item.blending_mode = OBSBlendingType[blending_mode]
+    if "scale_properties" in browser_source:
+        scale_filter = browser_source["scale_properties"]["scale_filter"]
+        scene_item.scale_filter = OBSScaleType[scale_filter]
+    obs_source.release()
+    scene.release()
+    source_json = source_to_json(scene_item)
+    if source_json is None:
+        raise ValueError("Source with type browser_source is not supported")
+    return {"source": source_json}
+
+
+@omu.endpoints.bind(endpoint_type=BROWSER_ADD)
+async def browser_add(browser_source: CreateBrowserRequest) -> CreateResponse:
+    browser_source["name"] = get_unique_name(browser_source["name"])
+    scene = get_scene(browser_source.get("scene"))
+    scene_width = scene.source.base_width
+    scene_height = scene.source.base_height
+    width, height = calculate_dimensions(
+        scene,
+        browser_source.get("width", scene_width),
+        browser_source.get("height", scene_height),
+    )
+    obs_data = {
+        "url": browser_source["url"],
+        "width": width,
+        "height": height,
+        "css": browser_source.get("css"),
+    }
+    obs_source = OBSSource.create(
+        "browser_source",
+        browser_source["name"],
+        OBSData.from_json(obs_data),
+    )
+    scene_item = scene.add(obs_source)
+    if "blend_properties" in browser_source:
+        blending_method = browser_source["blend_properties"]["blending_method"]
+        blending_mode = browser_source["blend_properties"]["blending_mode"]
+        scene_item.blending_method = OBSBlendingMethod[blending_method]
+        scene_item.blending_mode = OBSBlendingType[blending_mode]
+    if "scale_properties" in browser_source:
+        scale_filter = browser_source["scale_properties"]["scale_filter"]
+        scene_item.scale_filter = OBSScaleType[scale_filter]
+    obs_source.release()
+    scene.release()
+    source_json = source_to_json(scene_item)
+    if source_json is None:
+        raise ValueError("Source with type browser_source is not supported")
+    return {"source": source_json}
+
+
 @omu.endpoints.bind(endpoint_type=SOURCE_REMOVE_BY_NAME)
 async def source_remove_by_name(request: RemoveByNameRequest) -> RemoveResponse:
     obs_source = OBSSource.get_source_by_name(request["name"])
@@ -290,9 +413,7 @@ async def source_get_by_name(request: SourceGetByNameRequest) -> SourceJson:
     source_json = source_to_json(scene_item)
     try:
         if source_json is None:
-            raise ValueError(
-                f"Source with type {scene_item.source.id} is not supported"
-            )
+            raise ValueError(f"Source with type {scene_item.source.id} is not supported")
     finally:
         scene_item.release()
     return source_json
@@ -431,6 +552,42 @@ async def scene_set_current_by_uuid(
     return {}
 
 
+@omu.endpoints.bind(endpoint_type=SCENE_CREATE)
+async def scene_create(request: SceneCreateRequest) -> SceneCreateResponse:
+    scene = OBSScene.get_scene_by_name(request["name"])
+    if scene is not None:
+        scene.release()
+        raise ValueError(f"Scene with name {request['name']} already exists")
+    scene = OBSScene.create(request["name"])
+    response: SceneCreateResponse = {
+        "scene": {
+            "name": scene.source.name,
+            "uuid": scene.source.uuid,
+            "sources": [],
+        }
+    }
+    scene.release()
+    return response
+
+
+@omu.endpoints.bind(endpoint_type=SCREENSHOT_CREATE)
+async def screenshot_create(request: ScreenshotCreateRequest) -> ScreenshotCreateResponse:
+    OBS.frontend_take_screenshot()
+    return {}
+
+
+@omu.endpoints.bind(endpoint_type=SCREENSHOT_GET_LAST_BINARY)
+async def screenshot_get_last_binary(request: ScreenshotGetLastBinaryRequest) -> ScreenshotGetLastBinaryResponse:
+    screenshot_path = OBS.frontend_get_last_screenshot()
+    if screenshot_path is None:
+        return ScreenshotGetLastBinaryResponse(None)
+    try:
+        return ScreenshotGetLastBinaryResponse(screenshot_path.read_bytes())
+    except Exception as e:
+        logger.error(f"Failed to read screenshot: {e}")
+        return ScreenshotGetLastBinaryResponse(None)
+
+
 event_signal = omu.signals.get(EVENT_SIGNAL)
 
 
@@ -439,32 +596,65 @@ def on_event(event: OBSFrontendEvent):
     loop.create_task(event_signal.notify(event.name))
 
 
+@omu.network.event.connected.listen
+async def on_connected():
+    logger.info(f"Coneected to {omu.network.address.host}:{omu.network.address.port}")
+    print(f"[OMUAPPS] Connected to {omu.network.address.host}:{omu.network.address.port}")
+
+
 @omu.on_ready
 async def on_ready():
-    print("OBS Plugin is ready")
+    logger.info(f"OBS Plugin {VERSION} is ready!")
+    print(f"[OMUAPPS] OBS Plugin {VERSION} is ready!")
 
 
-thread = Thread(target=omu.loop.run_forever).start()
+_LOOP: asyncio.AbstractEventLoop | None = None
+_THREAD: threading.Thread | None = None
 
 
-def start():
+def setup_obs_plugin():
+    logger.info(f"Setting up OBS Plugin v{VERSION} at {Path.cwd()}")
+    global _LOOP
+    _LOOP = asyncio.new_event_loop()
+    asyncio.set_event_loop(_LOOP)
+
+    omu.set_loop(_LOOP)
     omu.loop.create_task(start_omu())
 
+    _LOOP.run_forever()
+
+    _LOOP.close()
+    _LOOP = None
+
+
+def script_load():
+    global _THREAD
+    _THREAD = threading.Thread(None, setup_obs_plugin, daemon=True)
+    _THREAD.start()
+
 
 async def start_omu():
     try:
         if omu.running:
             await omu.stop()
         await omu.start()
-    except Exception as e:
-        logger.opt(exception=e).warning("Failed to start OBS Plugin")
+    except Exception:
+        logger.warning("Failed to start OBS Plugin: {e}")
 
 
-def stop():
-    print("Stopping OBS Plugin")
-    omu.loop.create_task(stop_omu())
+def script_unload():
+    global _LOOP, _THREAD
+    if _LOOP is not None:
+        _LOOP.call_soon_threadsafe(lambda loop: loop.stop(), _LOOP)
+
+    if _THREAD is not None:
+        _THREAD.join(timeout=5)
+        _THREAD = None
 
 
 async def stop_omu():
     await omu.stop()
     print("OBS Plugin is stopped")
+
+
+omu.network.event.disconnected += stop_omu
